
from cloud_governance.common.utils.configs import INSTANCE_IDLE_DAYS
from cloud_governance.common.utils.utils import Utils
from cloud_governance.policy.helpers.azure.azure_policy_operations import AzurePolicyOperations


class InstanceIdle(AzurePolicyOperations):
    """
    This class stop the idle instances more than 4 days if it matches the required metrics
    CpuUtilization < 2 percent
    NetworkIN < 5k bytes
    NetworkOut < 5k bytes
    We trigger email if the instance is idle 2 days and stop the instance if it is idle 4 days
    """

    RESOURCE_ACTION = "Stop"

    def __init__(self):
        super().__init__()

    def run_policy_operations(self):
        """
        This method returns the running instances
        :return:
        :rtype:
        """
        vms_list = self._get_all_instances()
        running_vms = []
        for vm in vms_list:
            vm = vm.as_dict()
            status = self._get_instance_status(resource_id=vm.get("id"), vm_name=vm.get("name"))
            tags = vm.get("tags") if vm.get("tags", {}) else {}
            cluster_tag = self._get_cluster_tag(tags=tags)
            cleanup_result = False
            running_days = self.calculate_days(vm.get("time_created"))
            if Utils.contains_ignore_case(string=status, str1='running') and not cluster_tag and \
                    Utils.greater_than(val1=running_days, val2=INSTANCE_IDLE_DAYS) and \
                    self.verify_instance_idle(resource_id=vm.get("id")):
                cleanup_days = self.get_clean_up_days_count(tags=tags)
                cleanup_result = self.verify_and_delete_resource(resource_id=vm.get("id"), tags=tags,
                                                                 clean_up_days=cleanup_days)
                resource_data = self._get_es_schema(
                    resource_id=vm.get("name"),
                    skip_policy=self.get_skip_policy_value(tags=tags),
                    user=self.get_tag_name_from_tags(tags=tags, tag_name='User'),
                    launch_time=vm.get("time_created"),
                    resource_type=vm.get("hardware_profile", {}).get("vm_size"),
                    resource_state=status if not cleanup_result else 'Vm Stopped',
                    running_days=running_days, cleanup_days=cleanup_days,
                    dry_run=self._dry_run,
                    name=vm.get("name"),
                    resource_action=self.RESOURCE_ACTION,
                    region=vm.get("location"), cleanup_result=str(cleanup_result),
                    cloud_name=self._cloud_name
                )
                if self._force_delete and self._dry_run == 'no':
                    resource_data.update({'ForceDeleted': str(self._force_delete)})
                running_vms.append(resource_data)
            else:
                cleanup_days = 0
            if not cleanup_result:
                self.update_resource_day_count_tag(resource_id=vm.get("id"), cleanup_days=cleanup_days,
                                                   tags=tags)
        return running_vms



import os
import tempfile

import pandas as pd

from cloud_governance.common.google_drive.google_drive_operations import GoogleDriveOperations
from cloud_governance.main.environment_variables import environment_variables


class GCPOperations:

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__gsheet_operations = GoogleDriveOperations()
        self.__gsheet_id = self.__environment_variables_dict.get('SPREADSHEET_ID', '')

    def get_accounts_sheet(self, sheet_name: str, dir_path: str = None):
        with tempfile.TemporaryDirectory() as tmp_dir:
            dir_path = dir_path if dir_path else tmp_dir
            file_path = f'{dir_path}/{sheet_name}.csv'
            if not os.path.exists(file_path):
                self.__gsheet_operations.download_spreadsheet(spreadsheet_id=self.__gsheet_id,
                                                              sheet_name=sheet_name,
                                                              file_path=dir_path)
            accounts_df = pd.read_csv(file_path)
            return accounts_df




import os
import json
from github import Github


class GitLeaks:
    """
    This class search for github leaks
    """

    def __init__(self, git_access_token: str, git_repo: str, several_repos: bool = False,  report_file_name: str = "gitleaks_report.json", resource_file_name: str = "resources.json.gz"):
        self.__git_access_token = git_access_token
        self.__git_connect = Github(self.__git_access_token)
        self.__git_repo = git_repo
        self.__several_repos = several_repos
        self.__report_file_name = report_file_name
        self.__resource_file_name = resource_file_name
        self.__report_file_full_path = os.path.join(os.path.dirname(__file__), self.__report_file_name)
        self.__resources_file_full_path = os.path.join(os.path.dirname(__file__), self.__resource_file_name)

    def __delete_gitleaks_report(self):
        """
        This method clear report file
        @return:
        """
        report_file = self.__report_file_full_path
        if os.path.isfile(report_file):
            os.remove(report_file)

    # def __delete_gitleaks_resource(self):
    #     """
    #     This method clear resource file
    #     @return:
    #     """
    #     resource_file = self.__resources_file_full_path
    #     if os.path.isfile(resource_file):
    #         os.remove(resource_file)

    def __get_gitleaks_report(self):
        """
        This method returns dict report content
        """
        report_file = self.__report_file_full_path
        if os.path.isfile(report_file):
            json_file = open(report_file)
            json_str = json_file.read()
            json_data = json.loads(json_str)
            return json_data
        return None

    def scan_repo(self):
        """
        This method scan repo for leaks
        :return: The report content or None if empty
        """
        result_list = []
        if self.__several_repos:
            for repo in self.__git_connect.get_user().get_repos():
                os.system(f'gitleaks -r {self.__git_repo}/{repo.name} -o {self.__report_file_full_path} ')
                result = self.__get_gitleaks_report()
                if result:
                    result_list.extend(result)
                    self.__delete_gitleaks_report()
        else:
            os.system(f'gitleaks -r {self.__git_repo} -o {self.__report_file_full_path}')
            result = self.__get_gitleaks_report()
            if result:
                result_list.extend(result)
                self.__delete_gitleaks_report()
        return result_list

    # def save_results_to_s3(self, policy_output, region, policy):
    #     """
    #     This method save policy result to s3 with folder creation order by datetime
    #     @return:
    #     """
    #     s3operations = S3Operations(region_name=region)
    #     s3operations.save_results_to_s3(policy=policy, policy_output=policy_output, policy_result=self.__scan_repo())
        # with gzip.open(self.__resources_file_full_path, 'wt', encoding="ascii") as zipfile:
        #     json.dump(self.__scan_repo(), zipfile)
        # if 's3' in policy_output:
        #     s3_operations = S3Operations(region)
        #     date_key = datetime.datetime.now().strftime("%Y/%m/%d/%H")
        #     if '/' in policy_output:
        #         targets = policy_output.split('/')
        #         bucket = targets[2]
        #         logs = targets[3]
        #     s3_operations.upload_file(file_name_path=self.__resources_file_full_path, bucket=bucket, key=f'{logs}/{region}/{policy}/{date_key}', upload_file=self.__resource_file_name)
        # # save local
        # else:
        #     os.replace(self.__resources_file_full_path, fr'{policy_output}/{self.__resource_file_name}')
        # self.__delete_gitleaks_resource()



from abc import ABC
from datetime import datetime

from cloud_governance.cloud_resource_orchestration.clouds.common.abstract_cost_over_usage import AbstractCostOverUsage
from cloud_governance.cloud_resource_orchestration.utils.common_operations import string_equal_ignore_case
from cloud_governance.common.clouds.azure.compute.compute_operations import ComputeOperations
from cloud_governance.common.clouds.azure.cost_management.cost_management_operations import CostManagementOperations


class CostOverUsage(AbstractCostOverUsage, ABC):

    def __init__(self):
        super().__init__()
        self.__cost_mgmt_operations = CostManagementOperations()
        self.__compute_operations = ComputeOperations()
        self._subscription_id = self._environment_variables_dict.get('AZURE_SUBSCRIPTION_ID')
        self.__scope = f'subscriptions/{self._subscription_id}'

    def get_cost_management_object(self):
        """
        This method returns the object of cost_mgmt
        :return:
        :rtype:
        """
        return self.__cost_mgmt_operations

    def _verify_active_resources(self, tag_name: str, tag_value: str) -> bool:
        """
        This method verifies any active virtual instances in all regions by tag_name, tag_value
        :param tag_name:
        :type tag_name:
        :param tag_value:
        :type tag_value:
        :return:
        :rtype:
        """
        virtual_machines = self.__compute_operations.get_all_instances()
        for virtual_machine in virtual_machines:
            tags = virtual_machine.tags
            user = self.__compute_operations.check_tag_name(tags=tags, tag_name=tag_name)
            if string_equal_ignore_case(user, tag_value):
                return True
        return False

    def _get_cost_based_on_tag(self, start_date: str, end_date: str, tag_name: str, extra_filters: any = None,
                               extra_operation: str = 'And', granularity: str = None, forecast: bool = False, **kwargs):
        """
        This method returns the cost results based on the tag_name
        :param start_date:
        :type start_date:
        :param end_date:
        :type end_date:
        :param tag_name:
        :type tag_name:
        :param extra_filters:
        :type extra_filters:
        :param extra_operation:
        :type extra_operation:
        :param granularity:
        :type granularity:
        :param forecast:
        :type forecast:
        :return:
        :rtype:
        """
        start_date = datetime.strptime(start_date, "%Y-%m-%d")
        end_date = datetime.strptime(end_date, "%Y-%m-%d")
        if forecast:
            # Todo complete the forecast function
            results_by_time = self.__cost_mgmt_operations.get_forecast(start_date=start_date, end_date=end_date,
                                                                       granularity=granularity,
                                                                       grouping=['user'],
                                                                       scope=self.__scope, **kwargs
                                                                       )
        else:
            results_by_time = self.__cost_mgmt_operations.get_usage(scope=self.__scope, start_date=start_date,
                                                                    end_date=end_date, grouping=[tag_name],
                                                                    granularity=granularity, **kwargs)

            response = self.__cost_mgmt_operations.get_filter_data(cost_data=results_by_time, tag_name=tag_name)
            return response
        return {}



from datetime import datetime, timedelta

import boto3
from dateutil.relativedelta import relativedelta

from cloud_governance.common.utils.configs import HOURS_IN_SECONDS, HOURS_IN_DAY


class SavingsPlansOperations:
    """
    This class is for implementing savings plans methods
    """

    SAVINGS_PLANS = 'savingsPlans'
    NEXT_TOKEN = 'nextToken'
    DATE_TIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'
    DEFAULT_ROUND_DIGITS = 3

    def __init__(self, savings_plan_client=None):
        self.savings_plan_client = savings_plan_client if savings_plan_client else boto3.client(self.SAVINGS_PLANS.lower())

    def get_savings_plans_months(self, start_date: datetime, end_date: datetime):
        """
        This method returns the savings plans months ranges
        :param start_date:
        :param end_date:
        :return:
        """
        start_end_dates = []
        month_start_date = start_date
        while month_start_date <= end_date:
            # Calculate the month end date as the last day of the current month
            month_end_date = min(
                datetime(month_start_date.year, month_start_date.month, 1) + relativedelta(months=1) - relativedelta(
                    days=1), end_date)
            start_end_dates.append({'start': month_start_date, 'end': month_end_date, 'month': month_start_date.strftime("%b")})
            month_start_date += relativedelta(months=1)
            month_start_date = datetime(month_start_date.year, month_start_date.month, 1)
        return start_end_dates

    def get_savings_filter_data(self, savings_plans_list: list = None):
        """
        This method returns the savings plans filter values
        :return:
        """
        responses = []
        if not savings_plans_list:
            savings_plans_list = self.get_savings_plans_list()
        extract_values = ['savingsPlanId', 'state', 'savingsPlanType',
                          'paymentOption', 'productTypes', 'commitment', 'upfrontPaymentAmount',
                          'recurringPaymentAmount']
        for savings_plans in savings_plans_list:
            start_date = datetime.strptime(savings_plans.get('start'), self.DATE_TIME_FORMAT)
            end_date = datetime.strptime(savings_plans.get('end'), self.DATE_TIME_FORMAT)
            total_payment = savings_plans.get('upfrontPaymentAmount')
            daily_payment = float(total_payment) / (24 * 60)
            for date_range in self.get_savings_plans_months(start_date=start_date, end_date=end_date):
                start, end, month = date_range.get('start'), date_range.get('end'), date_range.get('month')
                savings_id = savings_plans.get('savingsPlanId')
                days = (end - start).days + 1
                month_payment = round(days * daily_payment, self.DEFAULT_ROUND_DIGITS)
                start, end = str(start.date()), str(end.date())
                monthly_savings_data = {'start': start, 'end': end, 'filter_date': f"{start}-{month}", 'index_id': f"{start}-{savings_id}", "CloudName": "AWS", 'SavingsMonthlyPayment': month_payment, 'CostType': 'savings_plans'}
                monthly_savings_data.update({value.title(): savings_plans.get(value) for value in extract_values})
                responses.append(monthly_savings_data)
        return responses

    def get_savings_plans_list(self, states: list = None, **kwargs):
        """
        This method returns the savings plans list
        :param states:
        :return:
        """
        results = {}
        if not states:
            states = ['active']
        kwargs.update({'states': states})
        if not kwargs.get('states'):
            kwargs.pop('states')
        response = self.savings_plan_client.describe_savings_plans(**kwargs)
        results[self.SAVINGS_PLANS] = response.get(self.SAVINGS_PLANS)
        while response.get(self.NEXT_TOKEN):
            response = self.savings_plan_client.describe_savings_plans(**kwargs)
            results[self.SAVINGS_PLANS].append(response.get(self.SAVINGS_PLANS))
        return results[self.SAVINGS_PLANS]

    def get_monthly_active_savings_plan_summary(self):
        """
        This method returns the monthly savings plans summary
        :return:
        :rtype:
        """
        savings_plans = self.get_savings_plans_list()
        monthly_sp_sum = {}
        for sp in savings_plans:
            start_date = datetime.fromisoformat(sp.get('start')[:-1])
            end_date = datetime.fromisoformat(sp.get('end')[:-1])
            commitment = float(sp.get('commitment', 0))
            monthly_commitments = self.__get_monthly_commitments(start_date=start_date, end_date=end_date,
                                                                 commitment=commitment)
            for month, monthly_commitment in monthly_commitments.items():
                monthly_sp_sum[month] = monthly_sp_sum.get(month, 0) + monthly_commitment
        return monthly_sp_sum

    def __get_monthly_commitments(self, start_date: datetime, end_date: datetime, commitment: float):
        """
        This method split the date rages to months
        :param start_date:
        :type start_date:
        :param end_date:
        :type end_date:
        :return:
        :rtype:
        """
        current_month = start_date
        monthly_ranges = {}
        while current_month <= end_date:
            month = current_month.month % 12 + 1
            year = current_month.year
            if current_month.month % 12 == 0:
                year = current_month.year + 1
                month = 1
            end_of_month = current_month.replace(year=year, month=month, day=1) - timedelta(days=1)
            end_of_month = min(end_of_month, end_date)
            total_seconds = (end_of_month - current_month).total_seconds()
            monthly_commitment = commitment * ((total_seconds / HOURS_IN_SECONDS) + HOURS_IN_DAY)
            monthly_ranges[str(current_month.date().replace(day=1))] = monthly_commitment
            if current_month.month == 12:
                current_month = datetime(current_month.year + 1, 1, 1)
            else:
                current_month = datetime(current_month.year, current_month.month + 1, 1)
        return monthly_ranges



import boto3
import typeguard

from cloud_governance.common.clouds.aws.ec2.ec2_operations import EC2Operations
from cloud_governance.common.jira.jira_operations import JiraOperations
from cloud_governance.common.ldap.ldap_search import LdapSearch
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp
from cloud_governance.main.environment_variables import environment_variables
from cloud_governance.policy.policy_operations.aws.tag_cluster.tag_cluster_operations import TagClusterOperations


class TagCROInstances:
    """
    This class manages the tagging instances which have the tag TicketId
    """
    KEY = 'Key'
    VALUE = 'Value'
    NA_USER = 'NA'
    EMPTY_USER = ''

    def __init__(self, region_name: str = ''):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__region_name = region_name if region_name else self.__environment_variables_dict.get('AWS_DEFAULT_REGION')
        self.__cro_resource_tag_name = self.__environment_variables_dict.get('CRO_RESOURCE_TAG_NAME')
        self.__ec2_client = boto3.client('ec2', region_name=self.__region_name)
        self.__ec2_operations = EC2Operations(region=self.__region_name)
        self.jira_operations = JiraOperations()
        self.__ldap_search = LdapSearch(ldap_host_name=self.__environment_variables_dict.get('LDAP_HOST_NAME', ''))
        self.__replace_user_names = self.__environment_variables_dict.get('CRO_REPLACED_USERNAMES')
        self.__tag_cluster_operations = TagClusterOperations(region=self.__region_name)

    @typeguard.typechecked
    @logger_time_stamp
    def __get_instance_volumes(self, block_device_mappings: list):
        """
        This method returns the instance volumes
        :param block_device_mappings:
        :return:
        """
        volumes_list = []
        for mapping in block_device_mappings:
            if mapping.get('Ebs').get('VolumeId'):
                volumes_list.append(mapping.get('Ebs').get('VolumeId'))
        return volumes_list

    @typeguard.typechecked
    @logger_time_stamp
    def __get_ldap_user_data(self, user: str, tag_name: str):
        """
        This method returns the ldap user tag_name
        :param user:
        :param tag_name:
        :return:
        """
        user_details = self.__ldap_search.get_user_details(user)
        if user_details:
            return user_details.get(tag_name)
        return self.NA_USER

    @logger_time_stamp
    def __tag_ticket_id_attach_instance(self, ticket_id: str, instance_id: str, volume_ids: list, user: str):
        """
        This method tag the instances which have the tag TicketId
        :param ticket_id:
        :param instance_id:
        :param volume_ids:
        :return:
        """
        ticket_id = ticket_id.split('-')[-1]
        ticket_description = self.jira_operations.get_issue_description(ticket_id=ticket_id, state='INPROGRESS')
        if ticket_description:
            duration = int(ticket_description.get('Days', 0))
            extended_duration = int(self.jira_operations.get_issue_sub_tasks_duration(ticket_id=ticket_id))
            duration += extended_duration
            estimated_cost = float(ticket_description.get('CostEstimation'))
            budget_extend_ticket_ids = self.jira_operations.get_budget_extend_tickets(ticket_id=ticket_id, ticket_state='closed')
            extended_budget = self.jira_operations.get_total_extend_budget(sub_ticket_ids=budget_extend_ticket_ids)
            estimated_cost = int(estimated_cost) + int(extended_budget)
            manager_approved = ticket_description.get('ApprovedManager')
            if not manager_approved:
                manager_approved = ticket_description.get('ManagerApprovalAddress')
            user_email = ticket_description.get('EmailAddress')
            user = user_email.split('@')[0]
            project = ticket_description.get('Project')
            tags = [{self.KEY: 'Duration', self.VALUE: str(duration)},
                    {self.KEY: 'EstimatedCost', self.VALUE: str(estimated_cost)},
                    {self.KEY: 'ApprovedManager', self.VALUE: manager_approved},
                    {self.KEY: 'Project', self.VALUE: project.upper()},
                    {self.KEY: 'Email', self.VALUE: user_email},
                    {self.KEY: self.__cro_resource_tag_name, self.VALUE: ticket_id},
                    {self.KEY: 'UserCRO', self.VALUE: user},
                    {self.KEY: 'Manager', self.VALUE: self.__get_ldap_user_data(user, "ManagerName").upper()},
                    {self.KEY: 'Owner', self.VALUE: self.__get_ldap_user_data(user, "FullName").upper()}]
            if user:
                tags.append({self.KEY: 'User', self.VALUE: user})
            self.__ec2_operations.tag_ec2_resources(client_method=self.__ec2_client.create_tags, resource_ids=[instance_id], tags=tags)
            if ticket_description.get('JiraStatus') != self.jira_operations.IN_PROGRESS:
                self.jira_operations.move_issue_state(ticket_id=ticket_id, state='inprogress')
            logger.info(f'Extra tags are added to the instances: {instance_id}, had an ticket_id: {ticket_id}')
            if volume_ids:
                try:
                    self.__ec2_operations.tag_ec2_resources(client_method=self.__ec2_client.create_tags, resource_ids=volume_ids, tags=tags)
                    logger.info(f'Tagged the instance: {instance_id} attached volumes {volume_ids}')
                except Exception as err:
                    logger.error(err)
            return True
        return False

    @logger_time_stamp
    def __tag_instances(self):
        """
        This method list the instances and tag the instances which have the tag TicketId
        :return:
        """
        ticket_id_instances = {}
        instances = self.__ec2_operations.get_ec2_instance_list()
        for resource in instances:
            instance_id = resource.get('InstanceId')
            user = self.__ec2_operations.get_tag_value_from_tags(tags=resource.get('Tags'), tag_name='User')
            if not user:
                user = self.__tag_cluster_operations.get_username(start_time=resource.get('LaunchTime'), resource_id=instance_id, resource_type='AWS::EC2::Instance', tags=resource.get('Tags'))
            ticket_id = self.__ec2_operations.get_tag_value_from_tags(tags=resource.get('Tags'), tag_name=self.__cro_resource_tag_name) if resource.get('Tags') else None
            if ticket_id:
                duration = self.__ec2_operations.get_tag_value_from_tags(tags=resource.get('Tags'), tag_name='Duration')
                cost_estimation = self.__ec2_operations.get_tag_value_from_tags(tags=resource.get('Tags'), tag_name='EstimatedCost')
                if not duration or not cost_estimation:
                    volume_ids = self.__get_instance_volumes(resource.get('BlockDeviceMappings'))
                    if self.__tag_ticket_id_attach_instance(ticket_id=ticket_id, instance_id=instance_id, volume_ids=volume_ids, user=user):
                        ticket_id_instances.setdefault(ticket_id, []).append(instance_id)
                user = self.__ec2_operations.get_tag_value_from_tags(tags=resource.get('Tags'), tag_name='User')
                tag_user = False
                if user in [*self.__replace_user_names, self.NA_USER]:
                    tag_user = True
                if tag_user:
                    user_cro = self.__ec2_operations.get_tag_value_from_tags(tags=resource.get('Tags'), tag_name='UserCRO')
                    if user_cro:
                        volume_ids = self.__get_instance_volumes(resource.get('BlockDeviceMappings'))
                        self.__ec2_operations.tag_ec2_resources(client_method=self.__ec2_client.create_tags, resource_ids=[instance_id, *volume_ids], tags=[{self.KEY: 'User', self.VALUE: user_cro}])
        return ticket_id_instances

    @logger_time_stamp
    def run(self):
        """
        This method run the tag instance methods
        :return:
        """
        return self.__tag_instances()



from cloud_governance.common.clouds.aws.cloudwatch.cloudwatch_operations import CloudWatchOperations
from cloud_governance.common.clouds.aws.ec2.ec2_operations import EC2Operations
from cloud_governance.common.clouds.aws.iam.iam_operations import IAMOperations
from cloud_governance.common.clouds.aws.price.resources_pricing import ResourcesPricing
from cloud_governance.common.clouds.aws.rds.rds_operations import RDSOperations
from cloud_governance.common.clouds.aws.s3.s3_operations import S3Operations
from cloud_governance.common.clouds.aws.utils.common_methods import get_boto3_client
from cloud_governance.common.clouds.aws.utils.utils import Utils
from cloud_governance.common.utils.configs import INSTANCE_IDLE_DAYS, DEFAULT_ROUND_DIGITS, TOTAL_BYTES_IN_KIB, \
    EC2_NAMESPACE, CLOUDWATCH_METRICS_AVAILABLE_DAYS
from cloud_governance.common.utils.utils import Utils
from cloud_governance.policy.helpers.abstract_policy_operations import AbstractPolicyOperations
from cloud_governance.common.logger.init_logger import logger


class AWSPolicyOperations(AbstractPolicyOperations):

    def __init__(self):
        super().__init__()
        self._region = self._environment_variables_dict.get('AWS_DEFAULT_REGION', 'us-east-2')
        self._cloud_name = 'AWS'
        self._ec2_client = get_boto3_client(client='ec2', region_name=self._region)
        self._s3_client = get_boto3_client('s3', region_name=self._region)
        self._iam_operations = IAMOperations()
        self._rds_operations = RDSOperations(region_name=self._region)
        self._s3operations = S3Operations(region_name=self._region)
        self._ec2_operations = EC2Operations(region=self._region)
        self._cloudwatch = CloudWatchOperations(region=self._region)
        self._resource_pricing = ResourcesPricing()

    def get_tag_name_from_tags(self, tags: list, tag_name: str) -> str:
        """
        This method returns the tag value from the tags
        :param tags:
        :type tags:
        :param tag_name:
        :type tag_name:
        :return:
        :rtype:
        """
        if tags:
            for tag in tags:
                if tag.get('Key').strip().lower() == tag_name.lower():
                    return tag.get('Value').strip()
        return ''

    def _delete_resource(self, resource_id: str):
        """
        This method deletes the resource by verifying the policy
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        action = "deleted"
        try:
            if self._policy == 's3_inactive':
                self._s3_client.delete_bucket(Bucket=resource_id)
            elif self._policy == 'empty_roles':
                self._iam_operations.delete_role(role_name=resource_id)
            elif self._policy == 'unattached_volume':
                self._ec2_client.delete_volume(VolumeId=resource_id)
            elif self._policy == 'ip_unattached':
                self._ec2_client.release_address(AllocationId=resource_id)
            elif self._policy == 'unused_nat_gateway':
                self._ec2_client.delete_nat_gateway(NatGatewayId=resource_id)
            elif self._policy == 'zombie_snapshots':
                self._ec2_client.delete_snapshot(SnapshotId=resource_id)
            elif self._policy == 'instance_run':
                self._ec2_client.stop_instances(InstanceIds=[resource_id])
                action = "Stopped"
            elif self._policy == 'database_idle':
                # @ Todo add the delete method after successful monitoring
                return False
            logger.info(f'{self._policy} {action}: {resource_id}')
        except Exception as err:
            logger.info(f'Exception raised: {err}: {resource_id}')

    def __remove_tag_key_aws(self, tags: list):
        """
        This method returns the tags that does not contain key startswith aws:
        :param tags:
        :type tags:
        :return:
        :rtype:
        """
        custom_tags = []
        for tag in tags:
            if not tag.get('Key').lower().startswith('aws'):
                custom_tags.append(tag)
        return custom_tags

    def _update_tag_value(self, tags: list, tag_name: str, tag_value: str):
        """
        This method returns the updated tag_list by adding the tag_name and tag_value to the tags
        @param tags:
        @param tag_name:
        @param tag_value:
        @return:
        """
        if self._dry_run == "yes":
            tag_value = 0
        tag_value = f'{self.CURRENT_DATE}@{tag_value}'
        found = False
        if tags:
            for tag in tags:
                if tag.get('Key') == tag_name:
                    if tag.get('Value').split("@")[0] != self.CURRENT_DATE:
                        tag['Value'] = tag_value
                    else:
                        if int(tag_value.split("@")[-1]) == 0 or int(tag_value.split("@")[-1]) == 1:
                            tag['Value'] = tag_value
                    found = True
        if not found:
            tags.append({'Key': tag_name, 'Value': tag_value})
        tags = self.__remove_tag_key_aws(tags=tags)
        return tags

    def update_resource_day_count_tag(self, resource_id: str, cleanup_days: int, tags: list,
                                      force_tag_update: str = ''):
        """
        This method updates the resource tags
        :param force_tag_update:
        :type force_tag_update:
        :param tags:
        :type tags:
        :param cleanup_days:
        :type cleanup_days:
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        tags = self._update_tag_value(tags=tags, tag_name='DaysCount', tag_value=str(cleanup_days))
        try:
            if self._policy == 's3_inactive':
                self._s3_client.put_bucket_tagging(Bucket=resource_id, Tagging={'TagSet': tags})
            elif self._policy == 'empty_roles':
                self._iam_operations.tag_role(role_name=resource_id, tags=tags)
            elif self._policy in ('ip_unattached', 'unused_nat_gateway', 'zombie_snapshots', 'unattached_volume',
                                  'instance_run', 'instance_idle'):
                self._ec2_client.create_tags(Resources=[resource_id], Tags=tags)
            elif self._policy == 'database_idle':
                self._rds_operations.add_tags_to_resource(resource_arn=resource_id, tags=tags)
        except Exception as err:
            logger.info(f'Exception raised: {err}: {resource_id}')

    def _get_all_instances(self):
        """
        This method updates the instance type count to the elasticsearch
        :return:
        :rtype:
        """
        instances = self._ec2_operations.get_ec2_instance_list()
        return instances

    def run_policy_operations(self):
        raise NotImplementedError("This method needs to be implemented")

    def _get_all_volumes(self, **kwargs) -> list:
        """
        This method returns the all volumes
        :return:
        :rtype:
        """
        volumes = self._ec2_operations.get_volumes(**kwargs)
        return volumes

    def _get_active_cluster_ids(self):
        """
        This method returns the active cluster id's
        :return:
        :rtype:
        """
        active_instances = self._ec2_operations.get_ec2_instance_list()
        cluster_ids = []
        for instance in active_instances:
            for tag in instance.get('Tags', []):
                if tag.get('Key', '').startswith('kubernetes.io/cluster'):
                    cluster_ids.append(tag.get('Key'))
                    break
        return cluster_ids

    def _get_global_active_cluster_ids(self):
        """
        This method returns the global active cluster ids
        :return:
        """
        cluster_ids = []
        active_regions = self._ec2_operations.get_active_regions()
        for region in active_regions:
            active_instances = self._ec2_operations.get_ec2_instance_list(
                ec2_client=get_boto3_client('ec2', region_name=region))
            for instance in active_instances:
                for tag in instance.get('Tags', []):
                    if tag.get('Key', '').startswith('kubernetes.io/cluster'):
                        cluster_ids.append(tag.get('Key'))
                        break
        return cluster_ids

    def _get_cluster_tag(self, tags: list):
        """
        This method returns the cluster_tag
        :return:
        :rtype:
        """
        if tags:
            for tag in tags:
                if tag.get('Key').startswith('kubernetes.io/cluster'):
                    return tag.get('Key')
        return ''

    def __get_aggregation_metrics_value(self, metrics: list, aggregation: str):
        """
        This method calculate the average of the metrics
        @param metrics:
        @param aggregation:
        @return:
        """
        metrics_result = 0
        for metric in metrics:
            metrics_values_sum = sum(metric['Values'])
            if Utils.equal_ignore_case(aggregation, 'average'):
                metrics_result += metrics_values_sum / len(metric['Values'])
            elif Utils.equal_ignore_case(aggregation, 'sum'):
                metrics_result += metrics_values_sum
        return round(metrics_result, DEFAULT_ROUND_DIGITS)

    def get_cpu_utilization_percentage_metric(self, resource_id: str, days: int = INSTANCE_IDLE_DAYS, **kwargs):
        """
        This method returns the average cpu utilization percentage
        :param resource_id:
        :type resource_id:
        :param days:
        :type days:
        :param kwargs:
        :type kwargs:
        :return:
        :rtype:
        """
        start_date, end_date = Utils.get_start_and_end_datetime(days=days)
        metrics = self._cloudwatch.get_metric_data(start_time=start_date, end_time=end_date, resource_id=resource_id,
                                                   resource_type='InstanceId', namespace=EC2_NAMESPACE,
                                                   metric_names={'CPUUtilization': 'Percent'},
                                                   statistic='Average')
        average_cpu_metrics_value = self.__get_aggregation_metrics_value(metrics.get('MetricDataResults', []),
                                                                         aggregation='average')
        return average_cpu_metrics_value

    def get_network_in_kib_metric(self, resource_id: str, days: int = INSTANCE_IDLE_DAYS, **kwargs):
        """
        This method returns the average network in bytes in KiB
        :param resource_id:
        :type resource_id:
        :param days:
        :type days:
        :param kwargs:
        :type kwargs:
        :return:
        :rtype:
        """
        start_date, end_date = Utils.get_start_and_end_datetime(days=days)
        metrics = self._cloudwatch.get_metric_data(start_time=start_date, end_time=end_date, resource_id=resource_id,
                                                   resource_type='InstanceId', namespace=EC2_NAMESPACE,
                                                   metric_names={'NetworkIn': 'Bytes'},
                                                   statistic='Average')
        average_network_in_bytes = self.__get_aggregation_metrics_value(metrics.get('MetricDataResults', []),
                                                                        aggregation='average')
        return round(average_network_in_bytes / TOTAL_BYTES_IN_KIB, DEFAULT_ROUND_DIGITS)

    def get_network_out_kib_metric(self, resource_id: str, days: int = INSTANCE_IDLE_DAYS, **kwargs):
        """
        This method returns the average network out bytes in KiB
        :param resource_id:
        :type resource_id:
        :param days:
        :type days:
        :param kwargs:
        :type kwargs:
        :return:
        :rtype:
        """
        start_date, end_date = Utils.get_start_and_end_datetime(days=days)
        metrics = self._cloudwatch.get_metric_data(start_time=start_date, end_time=end_date, resource_id=resource_id,
                                                   resource_type='InstanceId', namespace=EC2_NAMESPACE,
                                                   metric_names={'NetworkOut': 'Bytes'},
                                                   statistic='Average')
        average_network_out_bytes = self.__get_aggregation_metrics_value(metrics.get('MetricDataResults', []),
                                                                         aggregation='average')
        return round(average_network_out_bytes / TOTAL_BYTES_IN_KIB, DEFAULT_ROUND_DIGITS)

    def _get_ami_ids(self, **kwargs):
        """
        This method returns all image ids
        @return:
        """
        images = self._ec2_operations.get_images(**kwargs)
        image_ids = []
        for image in images:
            image_ids.append(image.get('ImageId'))
        return image_ids

    def __get_db_connection_status(self, resource_id: str, days: int = CLOUDWATCH_METRICS_AVAILABLE_DAYS):
        start_date, end_date = Utils.get_start_and_end_datetime(days=days)
        metrics = self._cloudwatch.get_metric_data(resource_id=resource_id, start_time=start_date, end_time=end_date,
                                                   resource_type='DBInstanceIdentifier',
                                                   metric_names={'DatabaseConnections': 'Count'},
                                                   namespace='AWS/RDS', statistic='Maximum'
                                                   )
        total_connections = self.__get_aggregation_metrics_value(metrics.get('MetricDataResults', []),
                                                                 aggregation='sum')
        return total_connections

    def is_database_idle(self, resource_id: str):
        """
        This method returns bool on verifying the database connections
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        total_connections = self.__get_db_connection_status(resource_id)
        return int(total_connections) == 0



import typeguard
from azure.core.paging import ItemPaged
from azure.core.polling import LROPoller
from azure.mgmt.resource import ResourceManagementClient
from azure.mgmt.resource.resources.v2022_09_01.models import ResourceGroup, GenericResourceExpanded, TagsResource, Tags

from cloud_governance.common.clouds.azure.common.common_operations import CommonOperations
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp


class ResourceGroupOperations(CommonOperations):

    def __init__(self):
        super().__init__()
        self.__resource_client = ResourceManagementClient(self._default_creds, subscription_id=self._subscription_id)

    @logger_time_stamp
    def get_all_resource_groups(self, **kwargs) -> [ResourceGroup]:
        """
        This method returns all resource groups present in azure subscription
        :return:
        """
        resource_groups_object: ItemPaged = self.__resource_client.resource_groups.list(**kwargs)
        resource_groups_list = self._item_paged_iterator(item_paged_object=resource_groups_object)
        return resource_groups_list

    @typeguard.typechecked
    @logger_time_stamp
    def get_all_resources(self, resource_group_name: str) -> [GenericResourceExpanded]:
        """
        This method returns all the resources in a resource_group
        :param resource_group_name:
        :type resource_group_name:
        :return:
        :rtype:
        """
        resources_list_object: ItemPaged = self.__resource_client.resources.list_by_resource_group(resource_group_name=resource_group_name)
        resources_list = self._item_paged_iterator(item_paged_object=resources_list_object)
        return resources_list

    def creates_or_updates_tags(self, resource_id: str, tags: dict):
        """
        This method creates or updates the tag on the specific resource
        :param tags:
        :param resource_id:
        :return:
        """
        status: LROPoller = self.__resource_client.tags.begin_create_or_update_at_scope(scope=resource_id,
                                                                                        parameters=self.__construct_tags(tags=tags))
        return status.done()

    def __construct_tags(self, tags: dict) -> TagsResource:
        """
        This method constructs the tags
        :param tags:
        :return:
        """
        tag_resource = TagsResource(properties=Tags(tags=tags))

        return tag_resource
